@@ -1,11 +1,11 @@
 use proc_macro::TokenStream;
-use syn::{ braced, token, parse_macro_input, Ident, Result, Token };
+use syn::{ braced, token, parse_macro_input, Ident, Result, Token, Type };
 use syn::parse::{ Parse, ParseStream };
 use syn::punctuated::Punctuated;
 use quote::{ quote };
 
 struct Machine {
-    name: Ident,
+    t: Type,
     #[allow(dead_code)]
     brace_token: token::Brace,
     states: Punctuated<StateDefinition, Token![,]>
@@ -29,7 +29,7 @@ impl Parse for Machine {
     fn parse(input: ParseStream) -> Result<Self> {
         let content;
         Ok(Machine {
-            name: input.parse()?,
+            t: input.parse()?,
             brace_token: braced!(content in input),
             states: content.parse_terminated(StateDefinition::parse)?,
         })
@@ -61,7 +61,7 @@ impl Parse for StateTransition {
 pub fn statemachine(input: TokenStream) -> TokenStream {
     let machine: Machine = parse_macro_input!(input as Machine);
 
-    let machine_name = &machine.name;
+    let t = &machine.t;
     let state_names = machine.states.iter().map(|x| &x.name);
     let state_names_copy = machine.states.iter().map(|x| &x.name);
     
@@ -80,19 +80,20 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             quote! {
                 #a
 
-                impl From<#machine_name<#state_name>> for #machine_name<#next_state_name> {
-                    fn from(val: #machine_name<#state_name>) -> #machine_name<#next_state_name> {
-                        #machine_name {
+                impl<T: OnChangeState> From<Machine<#state_name, T>> for Machine<#next_state_name, T> {
+                    fn from(val: Machine<#state_name, T>) -> Machine<#next_state_name, T> {
+                        Machine {
                             state: #next_state_name {
-                            }
+                            },
+                            t: val.t
                         }
                     }
                 }
 
-                impl #machine_name<#state_name> {
-                    pub fn #event(self) -> State {
-                        let next: #machine_name<#next_state_name> = self.into();
-                        #machine_name::<#next_state_name>::announce();
+                impl<T: OnChangeState> Machine<#state_name, T> {
+                    pub fn #event(self) -> State<T> {
+                        let next: Machine<#next_state_name, T> = self.into();
+                        next.t.on_change_state(stringify!(#state_name), stringify!(#next_state_name)).unwrap();
                         State::#next_state_name(next)
                     }
                 }
@@ -102,21 +103,17 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
         quote! {
             #a
 
-            #[derive(Clone, Copy)]
+            // #[derive(Clone, Copy)]
             pub struct #state_name {}
 
-            impl #machine_name<#state_name> {
-                pub fn new() -> Self {
-                    #machine_name::<#state_name>::announce();
-                    #machine_name {
+            impl<T: OnChangeState> Machine<#state_name, T> {
+                pub fn new(t: T) -> Self {
+                    Machine {
                         state: #state_name {
-                        }
+                        },
+                        t
                     }
                 }
-
-                fn announce() {
-                    println!("the machine currently assumes the {} state", stringify!(#state_name));
-                }
             }
 
             #transitions
@@ -124,21 +121,26 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
     });
 
     let tokens = quote!{
-        #[derive(Clone, Copy)]
-        pub struct #machine_name<S> {
-            state: S
+        pub trait OnChangeState {
+            fn on_change_state(&self, from: &str, to: &str) -> Result<(), ()>;
+        }
+
+        // #[derive(Clone, Copy)]
+        pub struct Machine<S, T: OnChangeState> {
+            state: S,
+            t: T
         }
 
         #state_structs
 
-        #[derive(Clone, Copy)]
-        pub enum State {
-            #(#state_names(#machine_name<#state_names>)),*
+        // #[derive(Clone, Copy)]
+        pub enum State<T: OnChangeState> {
+            #(#state_names(Machine<#state_names, T>)),*
         }
 
-        pub fn state_from_str(raw_state: &str) -> Option<State> {
+        pub fn state_from_str<T: OnChangeState>(raw_state: &str, t: T) -> Option<State<T>> {
             match raw_state {
-                #(stringify!(#state_names_copy) => Some(State::#state_names_copy(#machine_name::<#state_names_copy>::new()))),*,
+                #(stringify!(#state_names_copy) => Some(State::#state_names_copy(Machine::<#state_names_copy, T>::new(t)))),*,
                 _ => None
             }
         }
